/*
 * control.c, vi: tabstop=4
 * control panel of xvidcap
 *
 * Copyright (C) 1997,98 Rasca, Berlin
 * EMail: thron@gmx.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>		/* PATH_MAX */
#include <ctype.h>		/* isdigit() */
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/cursorfont.h>
#include <X11/Xmu/WinUtil.h>
#include <Xw/Button.h>
#include <Xw/Box.h>
#include <Xw/Toggle.h>
#ifdef HasZlib
#include <zlib.h>
#endif
#include "control.h"
#include "job.h"
#include "util.h"
#include "colors.h"
#include "capture.h"
#include "frame.h"
#include "main.h"
#include "xtoxwd.h"
#include "xtopnm.h"
/* #include "sound.h" */
#ifdef HasPNG
#include "xtopng.h"
#include "xtomng.h"
#endif
#include "xtojpg.h"
#include "xtoqtf.h"
#include "options.h"
#include "xbm/stop.xbm"
#include "xbm/pause.xbm"
#include "xbm/record.xbm"
#include "xbm/step.xbm"
#include "xbm/prev.xbm"
#include "xbm/next.xbm"
#include "xbm/select.xbm"
#include "xbm/animate.xbm"
/* #include "xbm/mkvideo.xbm" */
#include "xbm/edit.xbm"
/* #include "xbm/help.xbm" */
#include "xbm/move.xbm"

/* global
 */
static Job *jobp;
static Widget wFile, start, stop, pause, step, sel, animate, label;

/*
 */
static void
CbHelp (Widget w, XtPointer client_data, XtPointer call_data)
{
	if (jobp->flags & FLG_RUN_VERBOSE)
		printf ("calling '%s'\n", (char*) client_data);
	system ((char*)client_data);
}

/*
 * callback for edit button
 */
static void
CbEdit (Widget w, XtPointer client, XtPointer call)
{
	char cmd[PATH_MAX*2+4];
	char buf[PATH_MAX+1];
	int n;

	if (jobp->pic_no < 1)
		n = 0;
	else
		n = jobp->pic_no - 1;
	sprintf (buf, jobp->file, n);
	sprintf (cmd, (char*)client, buf);
	if (jobp->flags & FLG_RUN_VERBOSE)
		printf ("calling: '%s'\n", cmd);
	system (cmd);
}

/*
 */
static void
CbMkVideo (Widget w, XtPointer client, XtPointer call)
{
	char *fmt = (char *)client;
	char cmd[PATH_MAX*2+2];
	sprintf (cmd, fmt, jobp->file, jobp->start_no, jobp->pic_no-1,
		jobp->area->width, jobp->area->height, jobp->fps, jobp->time_per_frame);
	if (jobp->flags & FLG_RUN_VERBOSE)
		printf ("calling: '%s'\n", cmd);
	system (cmd);
}

/*
 */
static void
CbAnimate (Widget w, XtPointer client, XtPointer call)
{
	char file[PATH_MAX+1], buff[PATH_MAX*2+1], *p;
	char *cmd = (char *) client;
	int i = 0, n=0;

	p = jobp->file;
	while (*p) {
		if (*p == '%') {
			p++;
			sscanf (p, "%dd", &n);
			while (n) {
				file[i++] = '?';
				n--;
			}
			while (isdigit(*p))
				p++;
			p++;
		} else {
			file[i] = *p;
			p++;
			i++;
		}
	}
	file[i] = '\0';
	sprintf (buff, cmd, file, jobp->time_per_frame);
	if (jobp->flags & FLG_RUN_VERBOSE);
		printf ("calling: '%s'\n", buff);
	system (buff);
}

/*
 * change value of the label
 */
void
ChangeLabel (int pic_no)
{
	static char file[PATH_MAX+1];
	if (strstr (jobp->file, ".mng") || strstr (jobp->file, ".mov")) {
		sprintf (file, "%s[%04d]", jobp->file, pic_no);
	} else {
		sprintf (file, jobp->file, pic_no);
	}
	XtVaSetValues (label, XtNlabel, file, NULL);
}

/*
 */
static void
CbPrev (Widget w, XtPointer client, XtPointer call)
{
	Job *jobp = (Job *)client;
	if (jobp->pic_no > jobp->step)
		jobp->pic_no -= jobp->step;
	ChangeLabel (jobp->pic_no);
}

/*
 */
static void
CbNext (Widget w, XtPointer client, XtPointer call)
{
	Job *jobp = (Job *) client;
	jobp->pic_no += jobp->step;
	ChangeLabel (jobp->pic_no);
}

/*
 */
static void
CbReset (Widget w, XtPointer client, XtPointer call)
{
	if (jobp->pic_no != jobp->start_no) {
		jobp->pic_no = jobp->start_no;
		ChangeLabel (jobp->pic_no);
	}
}

/*
 */
static void
CbPause (Widget w, XtPointer client, XtPointer call)
{
	int state = 0;

	XtVaGetValues (w, XtNstate, &state, NULL);
#ifdef DEBUG
	printf ("CbPause(%d)\n", (int) state);
#endif
	if (state) {
		jobp->state |= VC_PAUSE;
		jobp->state &= ~VC_STOP;
		XtVaSetValues (stop, XtNstate, False, NULL);
		XtSetSensitive (stop, True);
		if (jobp->state & VC_REC)
			XtSetSensitive (step, True);
	} else {
		jobp->state &= ~VC_PAUSE;
		jobp->state &= ~VC_STEP;
		XtSetSensitive (step, False);
	}
}

/*
 * step mode, every click records one frame
 */
static void
CbStep (Widget w, XtPointer client, XtPointer call)
{
	Job *jobp = (Job *) client;
	if (!(jobp->state & (VC_PAUSE | VC_REC)))
		return;
	jobp->state |= VC_STEP;
}

/*
 */
void
CbQuit (Widget w, XtPointer client, XtPointer call)
{
	/* important for ?NG to sync the end of the file */
	if (jobp->state & VC_REC)
		if (jobp->clean)
			(*jobp->clean)(jobp);
	exit (0);
}

/*
 */
void
CbStop (Widget w, XtPointer client, XtPointer call)
{
	jobp->state = VC_STOP;
	XtSetSensitive (wFile, True);
	XtSetSensitive (stop, False);
	/* XtSetSensitive (start, True); */
	XtSetSensitive (step, False);
	XtSetSensitive (sel, True);
	XtVaSetValues (pause, XtNstate, False, NULL);
	XtVaSetValues (start, XtNstate, False, XtNsensitive, True, NULL);
	if (!(jobp->flags & FLG_RUN_VERBOSE))
		ChangeLabel(jobp->pic_no);
}

/*
 * timer callback for max_time
 */
void
TCbStop (XtPointer xtp, XtIntervalId id)
{
	CbStop (NULL, NULL, NULL);
}

/*
 */
static void
CbStart (Widget w, XtPointer client, XtPointer call)
{
	Job *jobp = (Job*) client;
	/* init some values for the new job
	 */
	jobp->state = (VC_REC | VC_START) | (jobp->state & VC_PAUSE);
#ifdef DEBUG
	printf ("CbStart() fps=%f time slot = %d ms\n", jobp->fps, jobp->time_per_frame);
#endif
	XtSetSensitive (w, False);
	XtSetSensitive (wFile, False);
	XtSetSensitive (stop, True);
	XtSetSensitive (sel, False);
	XtSetSensitive (pause, True);
	if (jobp->state & VC_PAUSE)
		XtSetSensitive (step, True);
	XtVaSetValues (stop, XtNstate, False, NULL);

	if (jobp->flags & FLG_MULTI_IMAGE)
		jobp->pic_no = jobp->start_no;
	if (jobp->max_time) {
		/* install a timer which stops recording
		 * we need milli secs ..
		 */
		XtAppAddTimeOut (XtWidgetToApplicationContext(w),
			(int)(jobp->max_time * 1000), (XtTimerCallbackProc)TCbStop, jobp);
	}
        /* call the timer function
         */
        XtAppAddTimeOut (XtWidgetToApplicationContext(w),
                         0, (XtTimerCallbackProc)jobp->capture, jobp);
}

/*
 * timer callback for start
 */
void
TCbStart (XtPointer xtp, XtIntervalId id)
{
	CbStart (start, xtp, NULL);
}

/*
 */
void
SelectSaveFunction (Visual *vis, int type)
{
	switch (type) {
		case CAP_PNM:
			switch (vis->class) {
				case TrueColor:
				case DirectColor:
					jobp->save = XImageToPPMC;
					break;
				case PseudoColor:
				case StaticColor:
					jobp->save = XImageToPPM8;
					break;
				case StaticGray:
				case GrayScale:
					printf ("Visual not supported\n");
					jobp->save = XImageToPGM;
					break;
			}
			break;
#ifdef HasPNG
		case CAP_PNG:
			switch (vis->class) {
				case TrueColor:
				case DirectColor:
					jobp->save = XImageToPNGC;
					break;
				case PseudoColor:
				case StaticColor:
				case StaticGray:
				case GrayScale:
					jobp->save = XImageToPNG8;
					break;
			}
			break;
		case CAP_MNG:
			switch (vis->class) {
				case TrueColor:
				case DirectColor:
					jobp->save = XImageToMNGC;
					break;
				case PseudoColor:
				case StaticColor:
				case StaticGray:
				case GrayScale:
					jobp->save = XImageToMNG8;
					break;
			}
			break;
#endif
#ifdef HasJPEG
		case CAP_JPG:
			switch (vis->class) {
				case TrueColor:
				case DirectColor:
					jobp->save = XImageToJPGC;
					break;
				case PseudoColor:
				case StaticColor:
					jobp->save = XImageToJPG8;
					break;
				case StaticGray:
					jobp->save = XImageToJPGG;
					break;
				case GrayScale:
					jobp->save = XImageToJPGG;
					printf ("Visual not supported!\n");
					break;
			}
			break;
#endif
		case CAP_QTR:
		case CAP_QTJ:
		case CAP_QTP:
			switch (vis->class) {
				case TrueColor:
				case DirectColor:
					jobp->save = XImageToQTFC;
					break;
				case StaticGray:
				case GrayScale:
					jobp->save = XImageToQTFG;
					break;
				default:
					printf ("Visual not supported!\n");
					exit (2);
					break;
			}
			break;
		default:	/* XWD */
			jobp->save = XImageToXWD;
			break;
	}
}

/*
 * select a window which should be captured
 */
static void
CbSelect (Widget w, XtPointer client, XtPointer call)
{
	Display *display = XtDisplay((Widget)client);
	Cursor cursor;
	Window root, target = None, temp = None;
	XEvent event;
	int buttons = 0;
	int x_down, y_down, x_up, y_up, x, y, pheight = 0;
	int width, height;
	Widget toplevel = (Widget)client;
	XGCValues gcv;
	GC gc;

	root = RootWindow (display, DefaultScreen(display));
	cursor = XCreateFontCursor (display, XC_crosshair);

	gcv.background = XBlackPixel (display, XDefaultScreen(display));
	gcv.foreground = XWhitePixel (display, XDefaultScreen(display));
	gcv.function = GXinvert;
	gcv.plane_mask = gcv.background ^ gcv.foreground;
	gcv.subwindow_mode = IncludeInferiors;
	gc = XCreateGC (display, root,
			GCBackground | GCForeground | GCFunction | GCPlaneMask | GCSubwindowMode, &gcv);
	/* grab the mouse
	 */
	if (XGrabPointer (display, root, False,
						PointerMotionMask|ButtonPressMask|ButtonReleaseMask,
						GrabModeSync, GrabModeAsync, root, cursor, CurrentTime)
			!= GrabSuccess) {
		printf ("Can't grab mouse!\n");
		return;
	}
	x_down = y_down = x_up = y_up = width = height = 0;

	while (buttons < 2) {
		/* allow pointer events */
		XAllowEvents(display, SyncPointer, CurrentTime);
		/* search in the queue for button events */
		XWindowEvent(display, root,
				PointerMotionMask|ButtonPressMask|ButtonReleaseMask, &event);
		switch (event.type) {
			case ButtonPress:
				x_down = event.xbutton.x;
				y_down = event.xbutton.y;
				target = event.xbutton.subwindow; /* window selected */
				if (target == None) {
					target = root;
				}
				buttons++;
				break;
			case ButtonRelease:
				x_up = event.xbutton.x;
				y_up = event.xbutton.y;
				buttons++;
				break;
			default:
				/* motion notify */
				if (buttons == 1) {
					if (width) {
						/* remove old frame */
						XDrawRectangle (display, root, gc, x, y, width, height);
					}
					/* button is pressed */
					if (x_down > event.xbutton.x) {
						width = x_down - event.xbutton.x + 1;
						x = event.xbutton.x;
					} else {
						width = event.xbutton.x - x_down + 1;
						x = x_down;
					}
					if (y_down > event.xbutton.y) {
						height = y_down - event.xbutton.y + 1;
						y = event.xbutton.y;
					} else {
						height = event.xbutton.y - y_down + 1;
						y = y_down;
					}
					XDrawRectangle (display, root, gc, x, y, width, height);
				}
				break;
		}
	}
	if (width) /* remove old frame */
		XDrawRectangle (display, root, gc, x, y, width, height);
	XUngrabPointer (display, CurrentTime);      /* Done with pointer */
	XFreeCursor (display, cursor);
	XFreeGC (display, gc);

	if ((x_down != x_up) && (y_down != y_up)) {
		/* an individual frame was selected
		 */
		if (x_up < x_down) {
			width = x_down - x_up + 2;
			x = x_up;
		} else {
			width = x_up - x_down + 2;
			x = x_down;
		}
		if (y_up < y_down) {
			height = y_down - y_up + 2;
			y = y_up;
		} else {
			height = y_up - y_down + 2;
			y = y_down;
		}
		if (target != root)
			target = XmuClientWindow (display, target);
		XGetWindowAttributes (display, target, &jobp->win_attr);
		jobp->win_attr.width = width;
		jobp->win_attr.height = height;
	} else {
		/* if (target == root) {
			printf ("Drag your mouse to select an area on the root window.\n");
			return;
		} */
		if (target != root) {
			/* get the real window */
			target = XmuClientWindow (display, target);
		}
		XGetWindowAttributes (display, target, &jobp->win_attr);
		XTranslateCoordinates(display, target, root, 0, 0, &x, &y, &temp);
	}

        adjust_grabbed_window(target);
	return;
}

/* 
   Removed that code from the previous function so it can be used
   by job_set_windowid().
 */

void
adjust_grabbed_window (Window target)
{
     int x, y, pheight = 0;
     Window root, temp = None;
     Widget toplevel = jobp->toplevel;     
     Display *display = XtDisplay(jobp->toplevel);

     root = RootWindow (display, DefaultScreen(display));
     XGetWindowAttributes (display, target, &jobp->win_attr);
     XTranslateCoordinates(display, target, root, 0, 0, &x, &y, &temp);
     jobp->win_attr.x = x;
     jobp->win_attr.y = y;
     jobp->windowid = target;

     /* we must move the panel a little bit, so get the height of
        the panel to adjust the y value */
     XtVaGetValues (toplevel, XtNheight, &pheight, NULL);

     /* add the space between the panel and the frame */
     /* QNC : pheight = pheight + FRAME_OFFSET; */
     pheight = pheight + jobp->offset_frame;
     if (IsFrameLocked()) {
          XtVaSetValues (toplevel, XtNx, x, XtNy, y - pheight, NULL);
     }
     ChangeFrame (x, y, jobp->win_attr.width, jobp->win_attr.height);
     if (jobp->flags & FLG_RUN_VERBOSE) {
          printf ("Selection geometry: %dx%d+%d+%d\n",
                  jobp->win_attr.width, jobp->win_attr.height, x, y);
     }
     jobp->ncolors = GetColors (display, &jobp->win_attr, &jobp->colors);
     if (jobp->get_colors) {
          if (jobp->color_table)
               free (jobp->color_table);
          jobp->color_table = (*jobp->get_colors)(jobp->colors, jobp->ncolors);
     }
     SelectSaveFunction (jobp->win_attr.visual, jobp->target);
#ifdef DEBUG
     printf ("new visual: %d\n", jobp->win_attr.visual->class);
#endif

}

/*
 */
void
MenuEH (Widget w, XtPointer xtp, XEvent *ev, Boolean *bo)
{
	Widget mw = (Widget) xtp;

#ifdef DEBUG
	printf ("MenuEH(): %s %d\n", XtName(w), ev->type);
#endif
	if (ev->type == ButtonRelease) {
		XtPopdown(mw);
		XUngrabPointer (XtDisplay(w), CurrentTime);
	}
}

/*
 * create control panel and make some inits..
 */
void
CreateControl ( Widget toplevel, char *file, int flags, int bpp,
				char *animate_cmd, char *mkvideo_cmd, char *edit_cmd,
				char *help_cmd)
{
	Widget box, file_menu, options, quit, help, prev, next, edit;
	Widget move, mkvideo, file_mbox;
	Display *display;
	Window root;
	static char info[PATH_MAX+1];
	Pixmap stop_bm, pause_bm, rec_bm, step_bm, prev_bm, next_bm;
	Pixmap select_bm, animate_bm, edit_bm, move_bm;

	display = XtDisplay (toplevel);
	root = RootWindow (display,DefaultScreen(display));

	jobp = job_init (flags, toplevel);

	jobp->area = GetArea();
	jobp->bpp = bpp;
	if (!XGetWindowAttributes (display, root, &jobp->win_attr))
		perror ("Can't get window attributes!\n");
	/* the default is to use the colormap of the root window
	 */
	jobp->ncolors = GetColors (display, &jobp->win_attr, &jobp->colors);

	sprintf (info, " %s     ", file);
	stop_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				stop_bits, stop_width, stop_height);
	pause_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				pause_bits, pause_width, pause_height);
	rec_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				record_bits, record_width, record_height);
	step_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				step_bits, step_width, step_height);
	prev_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				prev_bits, prev_width, prev_height);
	next_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				next_bits, next_width, next_height);
	select_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				select_bits, select_width, select_height);
	animate_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				animate_bits, animate_width, animate_height);
	edit_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				edit_bits, edit_width, edit_height);
	move_bm = XCreateBitmapFromData (XtDisplay(toplevel), root,
				move_bits, move_width, move_height);

	box =  XtVaCreateManagedWidget ("panel", xwBoxWidgetClass, toplevel,
				XtNorientation, XtorientHorizontal, NULL);

	/* file menu
	 */
	wFile = XtVaCreateManagedWidget ("file_mb", xwButtonWidgetClass, box,
					XtNmenuName, "file_menu",
					NULL);
	file_menu = XtVaCreatePopupShell ("file_menu", overrideShellWidgetClass,
					wFile, NULL);

	file_mbox = XtVaCreateManagedWidget ("file_mbox", xwBoxWidgetClass,
					file_menu, NULL);
	XtAddEventHandler (file_menu, ButtonReleaseMask|ButtonPressMask, False,
			MenuEH, (XtPointer)file_menu);

	options = XtVaCreateManagedWidget ("options", xwButtonWidgetClass,
					file_mbox, NULL);
	XtAddCallback (options, XtNcallback, CbOptions, (XtPointer)jobp);

	mkvideo = XtVaCreateManagedWidget ("mkvideo", xwButtonWidgetClass,
				file_mbox,
				NULL);
	XtAddCallback (mkvideo, XtNcallback, CbMkVideo, (XtPointer)mkvideo_cmd);

	help = XtVaCreateManagedWidget ("help", xwButtonWidgetClass, file_mbox,
				NULL);
	XtAddCallback (help, XtNcallback, CbHelp, (XtPointer)help_cmd);

	quit = XtVaCreateManagedWidget ("quit", xwButtonWidgetClass,
				file_mbox, NULL);
	XtAddCallback (quit, XtNcallback, CbQuit, NULL);
	/* XtAddEventHandler (quit, ButtonRelease, True, MenuEH, file_menu); */

	/* labels and bitmap buttons
	 */
    move = XtVaCreateManagedWidget ("move", xwToggleWidgetClass, box,
				XtNbitmap, move_bm, XtNstate, True,
				NULL);
    label = XtVaCreateManagedWidget ("label", xwButtonWidgetClass, box,
				XtNlabel, info,
				NULL);
    stop = XtVaCreateManagedWidget ("stop", xwToggleWidgetClass, box,
				XtNsensitive, False,
				XtNstate, True,
				XtNbitmap, stop_bm, NULL);
    pause = XtVaCreateManagedWidget ("pause", xwToggleWidgetClass, box,
				XtNbitmap, pause_bm, NULL);
    start= XtVaCreateManagedWidget ("start", xwToggleWidgetClass, box,
				XtNbitmap, rec_bm,
				NULL);
    step = XtVaCreateManagedWidget ("step", xwButtonWidgetClass, box,
				XtNsensitive, False,
				XtNbitmap, step_bm,
				NULL);
    prev = XtVaCreateManagedWidget ("prev", xwButtonWidgetClass, box,
				XtNbitmap, prev_bm,
				NULL);
    next = XtVaCreateManagedWidget ("next", xwButtonWidgetClass, box,
				XtNbitmap, next_bm,
				NULL);
    sel = XtVaCreateManagedWidget ("select", xwButtonWidgetClass, box,
				XtNbitmap, select_bm,
				NULL);
    animate = XtVaCreateManagedWidget ("animate", xwButtonWidgetClass, box,
				XtNbitmap, animate_bm,
				NULL);
    edit = XtVaCreateManagedWidget ("edit", xwButtonWidgetClass, box,
				XtNbitmap, edit_bm,
				NULL);

	XtVaSetValues (edit, XtNnext, wFile, XtNprev, animate, NULL);
	XtVaSetValues (animate, XtNnext, edit, XtNprev, sel, NULL);
	XtVaSetValues (sel, XtNnext, animate, XtNprev, next, NULL);
	XtVaSetValues (next, XtNnext, sel, XtNprev, prev, NULL);
	XtVaSetValues (prev, XtNnext, next, XtNprev, step, NULL);
	XtVaSetValues (step, XtNnext, prev, XtNprev, start, NULL);
	XtVaSetValues (start, XtNnext, step, XtNprev, pause, NULL);
	XtVaSetValues (pause, XtNnext, start, XtNprev, stop, NULL);
	XtVaSetValues (stop, XtNnext, pause, XtNprev, label, NULL);
	XtVaSetValues (label, XtNnext, stop, XtNprev, move, NULL);
	XtVaSetValues (move, XtNnext, label, XtNprev, wFile, NULL);
	XtVaSetValues (wFile, XtNnext, move, XtNprev, edit, NULL);

	XtAddCallback (move, XtNcallback, CbLock, (XtPointer)toplevel);
	XtAddCallback (label, XtNcallback, CbReset, (XtPointer)toplevel);
	XtAddCallback (stop, XtNcallback, CbStop, (XtPointer)toplevel);
	XtAddCallback (pause, XtNcallback, CbPause, (XtPointer)toplevel);
	XtAddCallback (start, XtNcallback, CbStart, (XtPointer)jobp);
	XtAddCallback (step, XtNcallback, CbStep, (XtPointer)jobp);
	XtAddCallback (prev, XtNcallback, CbPrev, (XtPointer)jobp);
	XtAddCallback (next, XtNcallback, CbNext, (XtPointer)jobp);
	XtAddCallback (sel, XtNcallback, CbSelect, (XtPointer)toplevel);
	XtAddCallback (animate, XtNcallback, CbAnimate, (XtPointer)animate_cmd);
	XtAddCallback (edit, XtNcallback, CbEdit, (XtPointer)edit_cmd);
}

/*
 * this is called after realizing the widgets above
 */
void
InitControl (char *file, int compress, float fps, float max_time,
				int max_frames, int start_at_no, int step,
				int quality, char *dev,
				char *snd, int rate, int size,
             char *script_file, char *result_file, Window windowid, 
             int delay, int offset_frame )
{
	/* 'jobp' is global */
        jobp->offset_frame = offset_frame;

#ifdef DEBUG
	printf ("InitControl() fps=%f\n", fps);
#endif
	/* find capture function */
	job_set_capture ();

	/* set frames per second */
	job_set_fps (fps);

	/* jpeg quality parameter */
	job_set_quality (quality);

	/* png compression */
	job_set_compression (compress);

	/* */
	job_set_sound_dev (snd, rate, size);

	/* */
	job_set_file (file);

        /* Set script and result files */
        job_set_script_file(script_file);
        job_set_result_file(result_file);
        job_set_windowid(windowid);

	jobp->max_time = max_time;
	jobp->max_frames = max_frames;
	jobp->start_no = start_at_no;
	jobp->pic_no = start_at_no;
	jobp->step = step;
	jobp->video_dev = dev;

	if (jobp->get_colors)
		jobp->color_table = (*jobp->get_colors)(jobp->colors, jobp->ncolors);

	SelectSaveFunction (jobp->win_attr.visual, jobp->target);
	ChangeLabel (jobp->pic_no);

        /* Schedule running the script */
        if ( delay >= 0 ) {
             printf("running the script in %d seconds\n", delay);
             XtAppAddTimeOut(XtWidgetToApplicationContext(jobp->toplevel),
                             1000 * delay, 
                             (XtTimerCallbackProc)TCbStart, 
                             jobp);
        }
}

