/*
 * frame.c, vi: tabstop=4
 *
 * Copyright (C) 1997 Rasca Gmelch, Berlin
 * EMail: thron@gmx.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include "Xw/Toggle.h"
#include "Xw/Label.h"
#include "frame.h"
#include "main.h"

/*
 * some globals
 */
static Widget
	top, left, right, bottom, blind = None;
static XRectangle rec;
static int lock = 1;

/*
 * change the position and the dimension of the capture frame
 */
void
ChangeFrame (int x, int y, int width, int height)
{
	int frame_width = FRAME_WIDTH;
	int max_width, max_height;

	/* we have to adjust it to viewable areas
	 */
	max_width = WidthOfScreen (XtScreen (top));
	max_height= HeightOfScreen(XtScreen (top));
#ifdef DEBUG
	printf ("ChangeFrame() screen = %dx%d selection=%dx%d\n",
			max_width, max_height, width, height);
#endif
	if (x < 0)
		x = 0;
	if (width > max_width)
		width = max_width;
	if (x + width > max_width)
		x = max_width - width;

	if (height > max_height)
		height = max_height;
	if (y + height > max_height)
		y = max_height - height;
	if (y < 0)
		y = 0;

	XtVaSetValues (top,
		XtNx, x - frame_width,
		XtNy, y - frame_width,
		XtNwidth, width + 2 * frame_width,
		XtNheight, frame_width,
		NULL);
	XtVaSetValues (bottom,
		XtNx, x - frame_width,
		XtNy, y + height,
		XtNwidth, width + 2 * frame_width,
		XtNheight, frame_width,
		NULL);
	XtVaSetValues (left,
		XtNx, x - frame_width,
		XtNy, y - frame_width,
		XtNwidth, frame_width,
		XtNheight, height + 2 * frame_width,
		NULL);
	XtVaSetValues (right,
		XtNx, x + width,
		XtNy, y - frame_width,
		XtNwidth, frame_width,
		XtNheight, height + 2 * frame_width,
		NULL);
	if (blind)
		XtVaSetValues (blind,
			XtNx, x,
			XtNy, y,
			XtNwidth, width,
			XtNheight, height,
			NULL);
	rec.x = x;
	rec.y = y;
	rec.width = width;
	rec.height= height;
}

/*
 * event handler
 * handle the capture frame, e.g. move, unmap, etc.
 */
void
MoveFrameEH (Widget parent, XtPointer xtp, XEvent *ev, Boolean *boo)
{
	Position x, y;
	Dimension pheight;
#ifdef DEBUG
	printf ("MoveFrameEH() event type = %d\n", ev->type);
#endif
	if (ev->type == ConfigureNotify) {
		if (!lock) {
			/* ignore it
			 */
			return;
		}
                /* QNC: I don't understand this so I removed it.
                XtVaGetValues (parent,
						XtNx, &x,
						XtNy, &y,
						XtNheight, &pheight,
						NULL);
                y += pheight + FRAME_OFFSET; 
		ChangeFrame (rec.x, rec.y, rec.width, rec.height);
                end of removal QNC */
	} else if ( ev->type == UnmapNotify ) {
		XtUnmanageChild (top);
		XtUnmanageChild (bottom);
		XtUnmanageChild (left);
		XtUnmanageChild (right);
		if (blind)
			XtUnmanageChild (blind);
	} else if ( ev->type == MapNotify ) {
		XtManageChild (top);
		XtManageChild (bottom);
		XtManageChild (left);
		XtManageChild (right);
		if (blind)
			XtManageChild (blind);
	} else if ( ev->type == FocusIn ) {
		XRaiseWindow (XtDisplay(top), XtWindow(top));
		XRaiseWindow (XtDisplay(top), XtWindow(bottom));
		XRaiseWindow (XtDisplay(top), XtWindow(left));
		XRaiseWindow (XtDisplay(top), XtWindow(right));
		if (blind)
			XRaiseWindow (XtDisplay(top), XtWindow(blind));
	}
}

/*
 * create the video frame
 */
void
CreateFrame (Widget parent, int width, int height, int flags)
{
	Position x, y;
	Dimension pwidth, pheight;

#ifdef DEBUG
	printf ("CreateFrame() %d %d\n", width, height);
#endif
	XtVaGetValues (parent, XtNx, &x, XtNy, &y, XtNwidth, &pwidth, XtNheight, &pheight, NULL);
	if (x < 0)
		x = 0;
	y += pheight + FRAME_OFFSET;
	if (y < 0)
		y = 0;
	rec.width = width;
	rec.height= height;
	rec.x = x;
	rec.y = y;
	top = XtVaCreatePopupShell ("top", overrideShellWidgetClass, parent,
			XtNx, x-FRAME_WIDTH,
			XtNy, y-FRAME_WIDTH,
			XtNwidth, width+2*FRAME_WIDTH,
			XtNheight, FRAME_WIDTH,
			NULL);
	bottom = XtVaCreatePopupShell ("bottom", overrideShellWidgetClass, parent,
			XtNx, x-FRAME_WIDTH,
			XtNy, y+height,
			XtNwidth, width+2*FRAME_WIDTH,
			XtNheight, FRAME_WIDTH,
			NULL);
	left= XtVaCreatePopupShell ("left", overrideShellWidgetClass, parent,
			XtNx, x-FRAME_WIDTH,
			XtNy, y-FRAME_WIDTH,
			XtNwidth, FRAME_WIDTH,
			XtNheight, height+2*FRAME_WIDTH,
			NULL);
	right= XtVaCreatePopupShell ("right", overrideShellWidgetClass, parent,
			XtNx, x+width,
			XtNy, y-FRAME_WIDTH,
			XtNwidth, FRAME_WIDTH,
			XtNheight, height+2*FRAME_WIDTH,
			NULL);
	if (flags & FLG_USE_V4L) {
		blind = XtVaCreatePopupShell ("blind", overrideShellWidgetClass, parent,
			XtNx, x+FRAME_WIDTH,
			XtNy, y+FRAME_WIDTH,
			XtNwidth, width,
			XtNheight, height,
			NULL);
		XtVaCreateManagedWidget ("text", xwLabelWidgetClass, blind,
			XtNlabel, "Source: Video4Linux", NULL);
		XtPopup(blind, XtGrabNone);
	}

	XtPopup(top, XtGrabNone);
	XtPopup(bottom, XtGrabNone);
	XtPopup(left, XtGrabNone);
	XtPopup(right, XtGrabNone);
	XtAddEventHandler (parent,
						StructureNotifyMask|FocusChangeMask|ExposureMask,
							False, MoveFrameEH, NULL);
}

/*
 * return a pointer to the current area enclosed by the frame
 */
XRectangle *
GetArea (void)
{
	return (&rec);
}

/*
 * lock or unlock the control panel for moving
 */
void
CbLock (Widget w, XtPointer client_data, XtPointer call_data)
{
	int val = XwIsSet (w);
#ifdef DEBUG
	printf ("CbMove (%d) %d \n", (int)w, (int)val);
#endif
	if (!val) {
		/* toggle released */
		lock = 0;
	} else {
		/* toggle pressed */
		lock = 1;
	}
}

/*
 */
int
IsFrameLocked ()
{
	return (lock);
}

