/*
 * main.c, vi: tabstop=4
 *
 * Copyright (C) 1997,98 Rasca, Berlin
 * EMail: thron@gmx.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#ifdef HasSharedMemory
#include <X11/extensions/XShm.h>
#endif
#ifdef HasDGA
#include <X11/extensions/xf86dga.h>
#endif
#ifdef HasVideo4Linux
#include <video.h>
#endif
#include "main.h"
#include "control.h"
#include "frame.h"
#include "fallback.h"

typedef struct {
	float fps;		/* frames per second */
	char *file;		/* file pattern */
	int verbose;	/* verbose level */
	int cap_width;
	int cap_height;
	float time;		/* time in seconds to record */
	int frames;
	int start_no;
	int step;
	int compress;
	int quality;
	int bpp;		/* bits per pixel */
	char *source;
	char *snddev;
	int sndrate;
	int sndsize;
	char *play_cmd;
	char *video_cmd;
	char *edit_cmd;
	char *help_cmd;
	Boolean use_clone;
        char *script_file;
        char *result_file;
        Window windowid;
        int delay;
        int offset_frame;
} AppData;
static AppData app;

void OptionDoneAction (Widget w, XEvent *ev, String *parms, Cardinal *num);
void OptionOpenAction (Widget w, XEvent *ev, String *parms, Cardinal *num);
static void QuitAppAction (Widget w, XEvent *ev, String *parms, Cardinal *num);

static XtActionsRec actionTable[] = {
	{"option_done", OptionDoneAction},
	{"option_open", OptionOpenAction},
	{"quit_app",	QuitAppAction	},
	};
/*
 */
void
usage (char *prog)
{
	printf ("Usage: %s, ver %s, (c) rasca, berlin 1997,98,99\n", prog, VERSION);
	printf ("[--fps #.#]         frames per second (float)\n");
	printf ("[--verbose #]       verbose level, '-v' is --verbose 1\n");
	printf ("[--time #.#]        time to record in seconds (float)\n");
	printf ("[--frames #]        frames to record, don't use it with --time\n");
	printf ("[--cap_size #x#]    size of the capture window (WIDTHxHEIGHT)\n");
	printf ("[--start_no #]      start number for the file names\n");
#ifdef HasZlib
	printf ("[--compress <0-9>]  save the frames compressed, '.gz' is added\n");
#endif
	printf ("[--use_clone]       use the CLON chunk for MNGs if needed\n");
	printf ("[--expand_to_24bit] expand 15/16bpp to 24bpp (only for PPM)\n");
#ifdef HasJPEG
	printf ("[--quality #]       quality value for JPEG files (100 - 0)\n");
#endif
	printf ("[--source <src>]    select input source: x11, shm, v4l[:<dev>]\n");
	printf ("[--file <file>]     file pattern, e.g. out%%03d.xwd\n");
        printf ("[--script <file>]   script file to process\n");
        printf ("[--result <file>]   result file to produce\n");
        printf ("[--window <windowid>] window to grab\n");
        printf ("[--offset_frame #]  vertical offset for the control panel\n");
        printf ("[--delay #]         start script automatically in # seconds\n");
	printf ("Supported output formats:\n");
	printf (" X window dump              (.xwd)\n");
	printf (" Portable anymap            (.pnm, .pgm, .ppm)\n");
#ifdef HasPNG
	printf (" Portable network graphics  (.png)\n");
	printf (" Multiple network graphics  (.mng)\n");
#endif
#ifdef HasJPEG
	printf (" Joint picture expert group (.jpg, .jpeg)\n");
#endif
	printf (" QuickTime                  (.mov)\n");
	exit (1);
}


/*
 * exit event handler
 */
static void
exitEH (Widget w, XtPointer xtp, XEvent *ev, Boolean *boo) {
	if ((ev->type == ClientMessage) &&
		(ev->xclient.format == 32) &&
		(ev->xclient.data.l[0] == *((Atom*)xtp)))
		exit (0);
}

#	define offset(x) XtOffsetOf(AppData, x)
static XtResource resources [] = {
{XtNfps, XtCFps, XtRFloat, sizeof(int), offset(fps),XtRImmediate, NULL},
{XtNfile, XtCFile,XtRString, sizeof(String), offset(file), XtRImmediate, NULL},
{XtNverbose, XtCVerbose,XtRInt,sizeof(int),offset(verbose),XtRImmediate,NULL},
{XtNcapWidth, XtCCapWidth,XtRInt,sizeof(int),offset(cap_width),XtRInt,0},
{XtNcapHeight, XtCCapHeight,XtRInt,sizeof(int),offset(cap_height),XtRInt,0},
{XtNtime, XtCTime, XtRFloat,sizeof(int),offset(time),XtRImmediate,NULL},
{XtNframes,XtCFrames,XtRInt,sizeof(int),offset(frames), XtRImmediate, NULL},
{XtNstartNo, XtCStartNo,XtRInt,sizeof(int),offset(start_no),XtRImmediate,NULL},
{XtNcomp, XtCComp,XtRInt,sizeof(int),offset(compress),XtRImmediate,NULL},
{XtNquality, XtCQuality,XtRInt,sizeof(int),offset(quality),XtRImmediate,NULL},
{XtNbpp, XtCBpp,XtRInt,sizeof(int),offset(bpp),XtRImmediate,NULL},
{XtNsource,XtCSource,XtRString,sizeof(String),offset(source),XtRImmediate,NULL},
{XtNsndDev,XtCSndDev,XtRString,sizeof(String),offset(snddev),XtRImmediate,NULL},
{XtNsndRate,XtCSndRate,XtRInt,sizeof(int),offset(sndrate),XtRImmediate,NULL},
{XtNsndSize,XtCSndSize,XtRInt,sizeof(int),offset(sndsize),XtRImmediate,NULL},
{XtNanimate,XtCAnimate,XtRString,sizeof(String),offset(play_cmd),XtRString, ""},
{XtNmkVideo,XtCMkVideo,XtRString,sizeof(String),offset(video_cmd),XtRString,""},
{XtNedit, XtCEdit,XtRString,sizeof(String),offset(edit_cmd),XtRImmediate, NULL},
{XtNhelp, XtCHelp,XtRString,sizeof(String),offset(help_cmd),XtRImmediate,NULL},
{"offsetFrame","OffsetFrame",XtRInt,sizeof(int),
      offset(offset_frame),XtRImmediate,NULL},
{"delayBeforeScript","DelayBeforeScript",XtRInt,sizeof(int),
      offset(delay),XtRImmediate,NULL},
};

/*
 * main()
 */
int
main (int argc, char **argv)
{
	char *program_name = "XVidcap";
        char *device = "/dev/video0";
	Widget toplevel;
	XtAppContext context;
	Display *dpy;
	Atom wm_delete;
	struct option options[] = {
		{"fps",      required_argument, NULL, 0},
		{"file",     required_argument, NULL, 0},
		{"verbose",  required_argument, NULL, 0},
		{"time",     required_argument, NULL, 0},
		{"frames",   required_argument, NULL, 0},
		{"cap_size", required_argument, NULL, 0},
		{"start_no", required_argument, NULL, 0},
		{"compress", required_argument, NULL, 0},
		{"use_clone", no_argument,		NULL, 0},
		{"expand_to_24bit", no_argument,NULL, 0},
		{"quality",  required_argument, NULL, 0},
		{"bpp", 	 required_argument, NULL, 0},
		{"source", 	 required_argument, NULL, 0},
		{"sync", 	 no_argument,		NULL, 0},
		{"step", 	 required_argument,	NULL, 0},
                {"script",   required_argument, NULL, 0},
                {"result",   required_argument, NULL, 0},
                {"window",   required_argument, NULL, 0},
                {"delay",    required_argument, NULL, 0},
                {"offset_frame",    required_argument, NULL, 0},
		{NULL, 0, NULL, 0},
	};
	int opt_index = 0, c, flags =0;
#ifdef HasDGA
	int dga_evb, dga_errb;
#endif

	app.use_clone = 0;
	app.quality = 99;
	app.source = NULL;
	app.step = 1;
	app.bpp = 0;
        app.script_file = NULL;
        app.result_file = NULL;
        app.windowid = 0;
        app.file = "out-%03d.xwd";

        { 
             char* xscript = "xscript";
             char *p = strstr(argv[0], xscript);
             if (    (p != NULL)
                  && (p+strlen(xscript) == '\0') ) {
                  program_name = "XScript";
             };
        }

	toplevel = XtVaAppInitialize ( &context, program_name,
				NULL, 0, &argc, argv, fallback, XtNinput, True, NULL);

	dpy = XtDisplay (toplevel);
	XtVaGetApplicationResources(toplevel, &app,
				resources, XtNumber(resources), NULL);
	XtAppAddActions (context,
				actionTable,sizeof(actionTable)/sizeof(XtActionsRec));

	if (app.source) {
		if (strstr (app.source, "v4l") != NULL) {
			flags |= FLG_USE_V4L;
			if (strchr (app.source, ':') != NULL) {
				device = strchr (app.source, ':') + 1;
			}
		} else if (strstr (app.source, "dga") != NULL)
			flags |= FLG_USE_DGA;
		else if (strstr (app.source, "shm") != NULL)
			flags |= FLG_USE_SHM;
	}
	while ((c = getopt_long (argc, argv, "v", options, &opt_index)) != -1) {
		switch (c) {
			case 0: /* is a long option */
				switch (opt_index) {
					case 0: /* frames per second */
						app.fps = atof (optarg);
						break;
					case 1: /* file for saving */
						app.file = strdup (optarg);
						break;
					case 2:
						app.verbose = atoi (optarg);
						break;
					case 3: /* time to record */
						app.time = atof (optarg);
						app.frames = 0;
						break;
					case 4:
						app.frames = atoi (optarg);
						app.time = 0;
						break;
					case 5:
						sscanf (optarg, "%dx%d",&app.cap_width,&app.cap_height);
						break;
					case 6:
						app.start_no = atoi (optarg);
						break;
					case 7:
						app.compress = atoi(optarg);
						break;
					case 8:
						app.use_clone = 1;
						break;
					case 9:
						flags |= FLG_EXPAND_TO_24BIT;
						break;
					case 10:
						app.quality = atoi(optarg);
						if ((app.quality < 0) || (app.quality > 100))
							app.quality = 75; /* reset to default */
						break;
					case 11:
						app.bpp = atoi(optarg);
						break;
					case 12:
						app.source = optarg;
						if (strstr (optarg, "v4l") != NULL) {
							flags &= ~FLG_SOURCE;
							flags |= FLG_USE_V4L;
							if (strchr (optarg, ':') != NULL) {
								device = strchr (optarg, ':')+1;
							}
						} else if (strstr (optarg, "dga") != NULL) {
							flags &= ~FLG_SOURCE;
							flags |= FLG_USE_DGA;
						} else if (strstr (optarg, "shm") != NULL) {
							flags &= ~FLG_SOURCE;
							flags |= FLG_USE_SHM;
						} else {
							flags &= ~FLG_SOURCE;
							printf ("using normal x server access ..\n");
						}
						break;
					case 13:
						flags |= FLG_SYNC;
						break;
					case 14:
						app.step = atoi(optarg);
						break;
                                     case 15: /* -script file */
                                          app.script_file = optarg;
                                          app.frames = 0;
                                          app.time = 0;
                                          printf("using script file %s\n",
                                                 app.script_file);
                                          break;
                                     case 16: /* -result file */
                                          app.result_file = optarg;
                                          printf("using result file %s\n",
                                                 app.result_file);
                                          break;
                                     case 17: /* -window */
                                          app.windowid = strtol(optarg, 0, 0);
                                          printf("using windowid 0x%lx\n",
                                                 app.windowid);
                                          break;
                                     case 18: /* -delay */
                                          if ( app.script_file ) {
                                               app.delay = strtol(optarg, 0, 0);
                                          } else {
                                               printf("Missing script\n");
                                          }
                                          break;
                                     case 19: /* -offset_frame */
                                          app.offset_frame = atoi(optarg);
                                          printf("Vertical offset: %d\n",
                                                 app.offset_frame);
                                          break;
					default:
						usage(argv[0]);
						break;
				}
				break;

			case 'v':
				app.verbose++;
				break;

			default:
				usage(argv[0]);
				break;
		}
	}
	if (argc != optind)
		usage(argv[0]);

#ifdef HasDGA
	if (!XF86DGAQueryExtension (XtDisplay(toplevel), &dga_evb, &dga_errb))
		flags &= ~FLG_USE_DGA;
	else {
		int flag = 0;
		XF86DGAQueryDirectVideo (dpy, XDefaultScreen(dpy), &flag);
		if ((flag & XF86DGADirectPresent) == 0) {
			flag &= ~FLG_USE_DGA;
			if (app.verbose) {
				printf ("xfdga: no direct present\n");
			}
		}
	}
#endif

#ifdef HasSharedMemory
	if (!XShmQueryExtension(XtDisplay(toplevel)))
		flags &= ~FLG_USE_SHM;
#endif
	if (app.verbose)
		flags |= FLG_RUN_VERBOSE;
	if (app.use_clone)
		flags |= FLG_MNG_CLONE;

	if (flags & FLG_USE_V4L) {
#ifndef HasVideo4Linux
		flags &= ~FLG_USE_V4L;
#else
		VIDEO *video;

		video = video_open (device, O_RDWR);
		if (!video)
			exit (3);
		if (app.verbose) {
			printf ("%s:\n cctm=%d channels=%d audios=%d maxwidth=%d "
					"maxheight=%d minwidth=%d minheight=%d\n",
					video->name, video->type&VID_TYPE_CAPTURE,
					video->channels, video->audios, video->maxwidth,
					video->maxheight, video->minwidth, video->minheight);
		}
		if (!(video->type & VID_TYPE_CAPTURE)) {
			fprintf (stderr, "video device can't capture to memory\n");
			exit (3);
		}
		app.cap_width = min(app.cap_width, video->maxwidth);
		app.cap_height = min(app.cap_height, video->maxheight);
		app.cap_width = max(app.cap_width, video->minwidth);
		app.cap_height = max(app.cap_height, video->minheight);
		video_close (video);
#endif
	}

#ifndef HasZlib
	app.compress = 0;
#else
	if (app.compress > 9)
		app.compress = 9;
#endif
	CreateControl (toplevel, app.file, flags, app.bpp,
					app.play_cmd, app.video_cmd, app.edit_cmd, app.help_cmd);

	CreateFrame (toplevel, app.cap_width, app.cap_height, flags & FLG_USE_V4L);
	XtRealizeWidget (toplevel);

	InitControl (app.file, app.compress, app.fps,
		app.time,app.frames,app.start_no, app.step, app.quality,
		device, app.snddev, app.sndrate, app.sndsize,
                     app.script_file, app.result_file, app.windowid, 
                     app.delay, app.offset_frame );

	if (app.verbose) {
		printf ("Current settings:\n");
		printf ("  frames per second = %f\n", app.fps);
		printf ("  file pattern      = %s\n", app.file);
		printf ("  verbose level     = %d\n", app.verbose);
		printf ("  frames to store   = %d\n", app.frames);
		printf ("  time to capture   = %f sec\n", app.time);
#ifdef HasZlib
		printf ("  compression level = %d\n", app.compress);
#endif
#ifdef HasJPEG
		printf ("  quality (jpeg)    = %d\n", app.quality);
#endif
		printf ("  input source      = %s (%d)\n", app.source,flags&FLG_SOURCE);
		printf ("  animate command   = %s\n", app.play_cmd);
		printf ("  make video command= %s\n", app.video_cmd);
		printf ("  edit frame command= %s\n", app.edit_cmd);
		printf ("  help command      = %s\n", app.help_cmd);
                printf ("  script file       = %s\n", app.script_file);
                printf ("  result file       = %s\n", app.result_file);
                printf ("  grabbed windowid  = %lx\n", app.windowid);
                printf ("  Y offset          = %d\n", app.offset_frame);
	}
	wm_delete = XInternAtom (XtDisplay(toplevel), "WM_DELETE_WINDOW", False);
	XSetWMProtocols (XtDisplay(toplevel), XtWindow(toplevel), &wm_delete, 1);
    XtAddEventHandler (toplevel, NoEventMask, True, exitEH, &wm_delete);

	XtAppMainLoop (context);
	return (0);
}

/*
 */
static void
QuitAppAction (Widget w, XEvent *ev, String *parms, Cardinal *num_parms)
{
	CbQuit (w, NULL, NULL);
}

/*
 */
/*static void
FileMenuAction (Widget w, XEvent *ev, String *parms, Cardinal *num_parms)
{
}*/

