# $Id: qnc.mk,v 1.22 2001/06/14 16:56:39 queinnec Exp $
# Copyright (C) 2001 by Christian.Queinnec@lip6.fr

# This is an addition to Xvidcap Makefile. It contains the rules to
# build, install and test xscript.
# Comments marked with #@perou: are durations measured on a Pentium 2, 450 MHz.
# That gives an indication of the expected duration for some tasks.

TAGS :
	etags *.[ch]

rebuild.for.distribution : 
	make clean ; ./configure
	${MAKE} TAGS
	${MAKE} rebuild.with.tags

rebuild.for.debugging : 
	${MAKE} rebuild.with.tags FLAGS="CDEBUGFLAGS='-g -DDEBUG'"

rebuild.with.tags :
	${MAKE} -f Makefile ${FLAGS}
	-cp -f XScript.ad /usr/X11R6/lib/X11/app-defaults/XScript
	-ln -s xvidcap xscript

xscript : xvidcap
	-ln -s xvidcap xscript

XSCRIPT	=	./xscript

test.xscript : rebuild.for.debugging 
	-rm -f cdls.t
	make cdls.t

# Run the cdls.xs script and obtain the cdls.t trace output file as well
# as all the cdls*.xwd files containing the captured windows.
cdls.t : cdls.xs
	${MAKE} run.rxvt
	-rm /tmp/coucou.tmp
	${XSCRIPT} --offset_frame 62 --fps 1 --script cdls.xs \
		--result cdls.t --delay 0 \
		--file 'cdls%04d.xwd' \
		--window `./id-of-window.pl '${XTERM} -bg'`
	@echo;echo '             'Close the rxvt window please.;echo
# NOTE: --fps 1 is really necessary otherwise xscript stops after sending
# the first character.

# NOTE: id-of-window.pl does not find the window id of netscape, use
# xwininfo instead and click to designate the interesting window.
# NOTE: when given a bad window-id, xscript does not complain and capture
# God knows what! Moreover the control window does not appear at all!
NETSCAPE_WINDOW_ID	=	`xwininfo <clic> | grep 'Window id:'`
netscape.t : netscape.xs
	${XSCRIPT} --offset_frame 62 --fps 1 --script netscape.xs \
		--result /tmp/netscape.t --delay 3 \
		--file '/tmp/ns%04d.xwd' \
		--window ${NETSCAPE_WINDOW_ID}

# NOTE: id-of-window.pl does not find the window id of drscheme, use
# xwininfo instead and click to designate the interesting window. Also
# click in the edition window to direct characters there (instead of
# the evaluation window).
# REMAINING PROBLEM: cr-lf seems to be inoperant with DrScheme.
DRSCHEME_WINDOW_ID	=	`xwininfo <clic> | grep 'Window id:'`
drscheme.t : drscheme.xs
	${XSCRIPT} --offset_frame 62 --fps 1 --script drscheme.xs \
		--result /tmp/drscheme.t --delay 0 \
		--file '/tmp/drs%04d.xwd' \
		--window ${DRSCHEME_WINDOW_ID}

TIMEOUT	=	90
XTERM	=	/usr/X11R6/bin/rxvt.bin
# In fact, rxvt is able to receive the characters events from xscript while
# xterm isn't. 
run.rxvt :
	${XTERM} -bg '#fdf5e6' -fg red & \
	    	sleep 3 ; (export ID=$$? ; sleep ${TIMEOUT} ; kill $$ID) &

# Combine the xwd files into an animated GIF:
cdls.GIF : cdls.t
	time perl buildAnimatedGif.pl cdls.GIF < cdls.t
#@perou: 102.60user 2.33system 1:50.57elapsed
#Resulting file is 3.6 Meg

# Then optimize it into a small animated gif:
cdls.gif : cdls.GIF
	time gifsicle --unoptimize cdls.GIF -O2 > cdls.gif
#@perou: 2.23user 0.12system 0:02.36elapsed
#Resulting file is 6516 bytes.

# Here are some sizes:
# perou% ll cdls*
# -rw-r--r--   1 queinnec spi        916458 Sep  4 15:54 cdls.GIF
# -rw-r--r--   1 queinnec spi          9598 Sep  4 15:54 cdls.gif
# -rw-r--r--   1 queinnec spi           387 Sep  4 15:53 cdls.t
# -r--r--r--   1 queinnec spi           401 Sep  4 15:52 cdls.xs
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:52 cdls0000.xwd
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:52 cdls0001.xwd
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:52 cdls0002.xwd
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:52 cdls0003.xwd
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:52 cdls0004.xwd
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:52 cdls0005.xwd
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:52 cdls0006.xwd
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:52 cdls0007.xwd
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:52 cdls0008.xwd
# -rw-r--r--   1 queinnec spi        314353 Sep  4 15:53 cdls0009.xwd
# To see the optimized version of cdls.gif: gifview cdls.gif
# NOTE: -dispose 1 as an option to convert within buildAnimatedGif
# leads to a smaller cdls.gif (the background is not saved).
# -rw-r--r--   1 queinnec spi          6664 Mar 27 18:34 cdls.gif
# Also compare gifsicle --info cdls.gif and gifsicle --info cdls.GIF

# NOTE: Your xwd files may be bigger if your X display is 24 bits-deep.

# Convert these frames into an animated gif with the same delay
# between frames (use 'convert' from ImageMagick).
#@perou: 36.24user 0.14system 0:36.51elapsed 99%CPU 

test.gif.animated :
	time perl playFilm.pl frm.gif < /tmp/SCRIPT1.t
	animate -verbose frm.gif

clean : clean.qnc
clean.qnc :
	-rm cdls[0-9]*.xwd cdls.GIF '.#Makefile' TAGS
	-rm -f out/cap* xvidcap.gz 
	-rm -f frm-*.xwd 
	-rm -f out-*.xwd 
	-rm -f *~
	-rm -f xscript
	-rm *.bak

# documentation
xscript.tex : xscript.bk
	LiSP2TeX xscript.bk > xscript.tex
xscript.dvi : xscript.tex
	-ln -s ../Cours/VideoC/videoc.gif .
	latex xscript.tex
xscript.ps : xscript.dvi
	dvips xscript.dvi -o xscript.ps
xscript.ps.gz : xscript.ps
	gzip -c xscript.ps > xscript.ps.gz

HEVEA		=	hevea
HEVEAFLAGS	=	-v -I ${HOME}/Hevea \
			-I /usr/local/lib/hevea \
			-I /usr/local/lib/hevea/html
xscript.html : xscript.aux xscript.dvi cdls.eps \
		../Cours/VideoScm/src/videoscm.hva
	-ln -s ../Cours/VideoScm/src/videoscm.css .
	${HEVEA} ${HEVEAFLAGS} xscript.tex

ckpt :
	$$HOME/perl/ckpt.prl .

# Utilities for distribution
# which xvidcap version is patched:

VERSION		=	1.0.9
ORIGINAL_DIR	=	/usr/local/src/

diff :
	-cd .. ; \
	diff -c -N -r ${ORIGINAL_DIR}/xvidcap-${VERSION} \
		xvidcap-${VERSION}.QNC \
		> `date +xvidcap-${VERSION}.${RELEASE}.patches`

STAMP	=	xscript
DATE	=	`date +%Y%h%d`
RELEASE	=	${STAMP}-${DATE}

distribution : build.tarball
distribution : update.spec build.rpm

build.tarball : cdls.t xscript.ps xscript.ps.gz xscript.html 
	$$HOME/perl/ckpt.prl
	make clean
	-ln -s ../perl/playFilm.pl buildAnimatedGif.pl
	-ln -s ../perl/id-of-window.pl .
	cd .. ; ln -s xscript ${RELEASE} ; export RELEASE="${RELEASE}" ; \
	tar -czhvf $$RELEASE.tgz --exclude $$RELEASE/Archive \
				  --exclude $$RELEASE/RCS \
				  --exclude $$RELEASE/cdls.GIF \
		$$RELEASE/?*.[ch] $$RELEASE/xvidcap.* \
		$$RELEASE/xbm $$RELEASE/xanim*diff \
		$$RELEASE/setup.* $$RELEASE/qnc.mk \
		$$RELEASE/ppm* $$RELEASE/out \
		$$RELEASE/mplay $$RELEASE/?*.pl $$RELEASE/?*.html \
		$$RELEASE/g* $$RELEASE/?*.png $$RELEASE/?*.in \
		$$RELEASE/config* $$RELEASE/cdls.* $$RELEASE/?*.xs \
		$$RELEASE/all2gif89a $$RELEASE/Xw $$RELEASE/README \
		$$RELEASE/?*.ad $$RELEASE/ChangeLog $$RELEASE/COPYING \
		$$RELEASE/Makefile* $$RELEASE/qnc.mk \
		$$RELEASE/xscript.ps $$RELEASE/xscript.html \
		$$RELEASE/videoscm.css 
	mv ../${RELEASE}.tgz .
	-rm ../${RELEASE}
	${MAKE} -f $$HOME/Makefile update.file STAMP=${STAMP} DATE=${DATE} \
		FILE=$$HOME/admin/Exportations
	${MAKE} -f $$HOME/Makefile update.file STAMP=${STAMP} DATE=${DATE} \
		FILE=$$HOME/Cours/VideoC/VideoC.html

RPMDIR          =       /usr/src/RPM

update.spec : 
	chmod u+w ${STAMP}.spec
	perl -i.bak ../perl/newHackSpec.prl ${STAMP}.spec
	cp ${STAMP}.spec ${RPMDIR}/SPECS/${RELEASE}.spec

build.rpm :
	@echo build rpm ...
	cp `ls -t ${STAMP}*.tgz | head -1` ${RPMDIR}/SOURCES/
	@echo
	@echo Becomes root and run:
	@echo '            ' cd ${RPMDIR}/SPECS/
	@echo '            ' rpm -ba ${RELEASE}.spec
	-xrun -sleep su -
	echo 
	echo Copying the fresh rpm
	echo
	cp ${RPMDIR}/RPMS/i''?86/${STAMP}-*.i''?86.rpm .
	echo Why this: cp ${RPMDIR}/SRPMS/${STAMP}-*.src.rpm .
	echo 
	echo Cleaning ... 
	echo
	-rm ${RPMDIR}/RPMS/i''?86/${STAMP}-*.i''?86.rpm
	-rm ${RPMDIR}/SRPMS/${STAMP}-*.src.rpm
	-rm ${RPMDIR}/BUILD/${STAMP}
	-rm ${RPMDIR}/SPECS/${STAMP}.spec
	@echo
	@echo Test fresh rpm...
	@echo 
	rpm -qpi `ls -t ${STAMP}-*.i''?86.rpm | head -1`

install.xscript : install.xscript.binaries
install.xscript : install.xscript.doc
install.xscript.binaries :
	-install -D --mode=555 --group=nobody -p \
		xscript /usr/local/bin/xscript
	-install -D --mode=444 --group=nobody -p \
		XScript.ad /usr/X11R6/lib/X11/app-defaults/XScript
install.xscript.doc :
	-rm -rf /usr/doc/xscript-*/
	-install -D --mode=444 --group=nobody -p \
		xscript.ps /usr/doc/${RELEASE}/xscript.ps
	-install -D --mode=444 --group=nobody -p \
		cdls.xs /usr/doc/${RELEASE}/cdls.xs
	-install -D --mode=444 --group=nobody -p \
		cdls.t /usr/doc/${RELEASE}/cdls.t
	-install -D --mode=444 --group=nobody -p \
		cdls.gif /usr/doc/${RELEASE}/cdls.gif

# end of qnc.mk
